import json

import pytest

from ninja_schema import ModelSchema, SchemaFactory, model_validator
from ninja_schema.errors import ConfigError
from tests.models import Event


class TestModelSchema:
    def test_schema_include_fields(self):
        class EventSchema(ModelSchema):
            class Config:
                model = Event
                include = "__all__"

        assert EventSchema.schema() == {
            "title": "EventSchema",
            "type": "object",
            "properties": {
                "id": {"title": "Id", "extra": {}, "type": "integer"},
                "title": {"title": "Title", "maxLength": 100, "type": "string"},
                "category_id": {"title": "Category", "type": "integer"},
                "start_date": {
                    "title": "Start Date",
                    "type": "string",
                    "format": "date",
                },
                "end_date": {"title": "End Date", "type": "string", "format": "date"},
            },
            "required": ["title", "start_date", "end_date"],
        }

        class Event2Schema(ModelSchema):
            class Config:
                model = Event
                include = ["title", "start_date", "end_date"]

        assert Event2Schema.schema() == {
            "title": "Event2Schema",
            "type": "object",
            "properties": {
                "title": {"title": "Title", "maxLength": 100, "type": "string"},
                "start_date": {
                    "title": "Start Date",
                    "type": "string",
                    "format": "date",
                },
                "end_date": {"title": "End Date", "type": "string", "format": "date"},
            },
            "required": ["title", "start_date", "end_date"],
        }

    def test_schema_depth(self):
        class EventDepthSchema(ModelSchema):
            class Config:
                model = Event
                include = "__all__"
                depth = 1

        assert EventDepthSchema.schema() == {
            "title": "EventDepthSchema",
            "type": "object",
            "properties": {
                "id": {"title": "Id", "extra": {}, "type": "integer"},
                "title": {"title": "Title", "maxLength": 100, "type": "string"},
                "category": {
                    "title": "Category",
                    "allOf": [{"$ref": "#/definitions/Category"}],
                },
                "start_date": {
                    "title": "Start Date",
                    "type": "string",
                    "format": "date",
                },
                "end_date": {"title": "End Date", "type": "string", "format": "date"},
            },
            "required": ["title", "start_date", "end_date"],
            "definitions": {
                "Category": {
                    "title": "Category",
                    "type": "object",
                    "properties": {
                        "id": {"title": "Id", "extra": {}, "type": "integer"},
                        "name": {"title": "Name", "maxLength": 100, "type": "string"},
                        "start_date": {
                            "title": "Start Date",
                            "type": "string",
                            "format": "date",
                        },
                        "end_date": {
                            "title": "End Date",
                            "type": "string",
                            "format": "date",
                        },
                    },
                    "required": ["name", "start_date", "end_date"],
                }
            },
        }

    def test_schema_exclude_fields(self):
        class Event3Schema(ModelSchema):
            class Config:
                model = Event
                exclude = ["id", "category"]

        assert Event3Schema.schema() == {
            "title": "Event3Schema",
            "type": "object",
            "properties": {
                "title": {"title": "Title", "maxLength": 100, "type": "string"},
                "start_date": {
                    "title": "Start Date",
                    "type": "string",
                    "format": "date",
                },
                "end_date": {"title": "End Date", "type": "string", "format": "date"},
            },
            "required": ["title", "start_date", "end_date"],
        }

    def test_schema_optional_fields(self):
        class Event4Schema(ModelSchema):
            class Config:
                model = Event
                include = "__all__"
                optional = "__all__"

        assert Event4Schema.schema() == {
            "title": "Event4Schema",
            "type": "object",
            "properties": {
                "id": {"title": "Id", "extra": {}, "type": "integer"},
                "title": {
                    "title": "Title",
                    "extra": {},
                    "maxLength": 100,
                    "type": "string",
                },
                "category_id": {"title": "Category", "extra": {}, "type": "integer"},
                "start_date": {
                    "title": "Start Date",
                    "extra": {},
                    "type": "string",
                    "format": "date",
                },
                "end_date": {
                    "title": "End Date",
                    "extra": {},
                    "type": "string",
                    "format": "date",
                },
            },
        }

        class Event5Schema(ModelSchema):
            class Config:
                model = Event
                include = ["id", "title", "start_date"]
                optional = [
                    "start_date",
                ]

        assert Event5Schema.schema() == {
            "title": "Event5Schema",
            "type": "object",
            "properties": {
                "id": {"title": "Id", "type": "integer"},
                "title": {"title": "Title", "maxLength": 100, "type": "string"},
                "start_date": {
                    "title": "Start Date",
                    "extra": {},
                    "type": "string",
                    "format": "date",
                },
            },
            "required": [
                "id",
                "title",
            ],
        }

    def test_schema_custom_fields(self):
        class Event6Schema(ModelSchema):
            custom_field1: str
            custom_field2: int = 1
            custom_field3 = ""
            _custom_field4 = []  # ignored by pydantic

            class Config:
                model = Event
                exclude = ["id", "category"]

        assert Event6Schema.schema() == {
            "title": "Event6Schema",
            "type": "object",
            "properties": {
                "title": {"title": "Title", "maxLength": 100, "type": "string"},
                "start_date": {
                    "title": "Start Date",
                    "type": "string",
                    "format": "date",
                },
                "end_date": {"title": "End Date", "type": "string", "format": "date"},
                "custom_field1": {"title": "Custom Field1", "type": "string"},
                "custom_field3": {
                    "title": "Custom Field3",
                    "default": "",
                    "type": "string",
                },
                "custom_field2": {
                    "title": "Custom Field2",
                    "default": 1,
                    "type": "integer",
                },
            },
            "required": ["custom_field1", "title", "start_date", "end_date"],
        }

    def test_model_validator(self):
        class EventSchema(ModelSchema):
            class Config:
                model = Event
                include = [
                    "title",
                    "start_date",
                ]

            @model_validator("title")
            def validate_title(cls, value):
                return f"{value} - value cleaned"

        event = EventSchema(start_date="2021-06-12", title="PyConf 2021")
        assert "value cleaned" in event.title

        class Event2Schema(ModelSchema):
            custom_field: str

            class Config:
                model = Event
                include = [
                    "title",
                    "start_date",
                ]

            @model_validator("title", "custom_field")
            def validate_title(cls, value):
                return f"{value} - value cleaned"

        event2 = Event2Schema(
            start_date="2021-06-12",
            title="PyConf 2021",
            custom_field="some custom name",
        )
        assert "value cleaned" in event2.title
        assert "value cleaned" in event2.custom_field

    def test_invalid_fields_inputs(self):
        with pytest.raises(ConfigError):

            class Event1Schema(ModelSchema):
                class Config:
                    model = Event
                    include = ["xy", "yz"]

        with pytest.raises(ConfigError):

            class Event2Schema(ModelSchema):
                class Config:
                    model = Event
                    exclude = ["xy", "yz"]

        with pytest.raises(ConfigError):

            class Event3Schema(ModelSchema):
                class Config:
                    model = Event
                    optional = ["xy", "yz"]

    def test_model_validator_not_used(self):
        with pytest.raises(ConfigError):

            class Event1Schema(ModelSchema):
                class Config:
                    model = Event
                    exclude = [
                        "title",
                    ]

                @model_validator("title")
                def validate_title(cls, value):
                    return f"{value} - value cleaned"

        with pytest.raises(ConfigError):

            class Event2Schema(ModelSchema):
                class Config:
                    model = Event
                    include = [
                        "title",
                    ]

                @model_validator("title", "invalid_field")
                def validate_title(cls, value):
                    return f"{value} - value cleaned"

    def test_factory_functions(self):
        event_schema = SchemaFactory.create_schema(model=Event, name="EventSchema")
        print(json.dumps(event_schema.schema(), sort_keys=False, indent=4))
        assert event_schema.schema() == {
            "title": "EventSchema",
            "type": "object",
            "properties": {
                "id": {"title": "Id", "extra": {}, "type": "integer"},
                "title": {"title": "Title", "maxLength": 100, "type": "string"},
                "category_id": {"title": "Category", "type": "integer"},
                "start_date": {
                    "title": "Start Date",
                    "type": "string",
                    "format": "date",
                },
                "end_date": {"title": "End Date", "type": "string", "format": "date"},
            },
            "required": ["title", "start_date", "end_date"],
        }

    def get_new_event(self, title):
        event = Event(title=title)
        event.save()
        return event

    @pytest.mark.django_db
    def test_getter_functions(self):
        class EventSchema(ModelSchema):
            class Config:
                model = Event
                include = ["title", "category", "id"]

        event = self.get_new_event(title="PyConf")
        json_event = EventSchema.from_orm(event)

        assert json_event.dict() == {"id": 1, "title": "PyConf", "category": None}
        json_event.title = "PyConf Updated"

        json_event.apply_to_model(event)
        assert event.title == "PyConf Updated"



"""Django Schema - Builds Pydantic Schemas from Django Models"""

__version__ = "0.12.2"

from .orm.factory import SchemaFactory
from .orm.model_schema import ModelSchema
from .orm.model_validators import model_validator
from .orm.schema import Schema

__all__ = ["SchemaFactory", "Schema", "ModelSchema", "model_validator"]



import inspect
from typing import Type

from django.db import models
from django.db.models import Model


def is_valid_django_model(model: Type[Model]) -> bool:
    return is_valid_class(model) and issubclass(model, models.Model)


def is_valid_class(klass: type) -> bool:
    return inspect.isclass(klass)



from typing import TYPE_CHECKING, Dict, Tuple, Type, Union

from django.db.models import Model

from .schema import Schema
from .utils.utils import is_valid_class, is_valid_django_model

if TYPE_CHECKING:
    from .model_schema import ModelSchema

__all__ = ["SchemaRegister", "registry"]


class SchemaRegisterBorg:
    _shared_state: Dict[str, Dict] = {}

    def __init__(self) -> None:
        self.__dict__ = self._shared_state


class SchemaRegister(SchemaRegisterBorg):
    schemas: Dict[Type[Model], Union[Type["ModelSchema"], Type[Schema]]]
    fields: Dict[str, Tuple]

    def __init__(self) -> None:
        SchemaRegisterBorg.__init__(self)
        if not hasattr(self, "schemas"):
            self._shared_state.update(schemas={}, fields={})

    def register_model(self, model: Type[Model], schema: Type["ModelSchema"]) -> None:
        from .model_schema import ModelSchema

        assert is_valid_class(schema) and issubclass(
            schema, (ModelSchema,)
        ), "Only Schema can be" 'registered, received "{}"'.format(schema.__name__)
        assert is_valid_django_model(
            model
        ), "Only Django Models are allowed. {}".format(model.__name__)
        # TODO: register model as module_name.model_name
        self.register_schema(model, schema)

    def register_schema(
        self, name: Type[Model], schema: Union[Type["ModelSchema"], Type[Schema]]
    ) -> None:
        self.schemas[name] = schema

    def get_model_schema(
        self, model: Type[Model]
    ) -> Union[Type["ModelSchema"], Type[Schema], None]:
        if model in self.schemas:
            return self.schemas[model]
        return None


registry = SchemaRegister()



import datetime
import re
from enum import Enum
from functools import singledispatch
from typing import (
    TYPE_CHECKING,
    Any,
    Iterable,
    Iterator,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    no_type_check,
)
from uuid import UUID

from django.db import models
from django.db.models.fields import Field
from django.utils.encoding import force_str
from pydantic import AnyUrl, EmailStr, Json
from pydantic.fields import FieldInfo, Undefined

from ...compat import ArrayField, HStoreField, JSONField, RangeField
from ...types import DictStrAny
from ..factory import SchemaFactory
from ..schema_registry import SchemaRegister, registry as global_registry

if TYPE_CHECKING:
    from ..model_schema import ModelSchema


TModel = TypeVar("TModel")

NAME_PATTERN = r"^[_a-zA-Z][_a-zA-Z0-9]*$"
COMPILED_NAME_PATTERN = re.compile(NAME_PATTERN)


def assert_valid_name(name: str) -> None:
    """Helper to assert that provided names are valid."""
    assert COMPILED_NAME_PATTERN.match(
        name
    ), 'Names must match /{}/ but "{}" does not.'.format(NAME_PATTERN, name)


def convert_choice_name(name: str) -> str:
    name = force_str(name)
    try:
        assert_valid_name(name)
    except AssertionError:
        name = "A_%s" % name
    return name


def get_choices(
    choices: Iterable[Union[Tuple[Any, Any], Tuple[str, Iterable[Tuple[Any, Any]]]]]
) -> Iterator[Tuple[str, str, str]]:
    for value, help_text in choices:
        if isinstance(help_text, (tuple, list)):
            for choice in get_choices(help_text):
                yield choice
        else:
            name = convert_choice_name(value)
            description = force_str(help_text)
            yield name, value, description


class FieldConversionProps:
    description: str
    blank: bool
    is_null: bool
    max_length: int
    alias: str
    title: str

    def __init__(self, field: Field):
        data = {}
        field_options = field.deconstruct()[3]  # 3 are the keywords

        data["description"] = force_str(
            getattr(field, "help_text", field.verbose_name)
        ).strip()
        data["title"] = field.verbose_name.title()

        if not field.is_relation:
            data["blank"] = field_options.get("blank", False)
            data["is_null"] = field_options.get("null", False)
            data["max_length"] = field_options.get("max_length")
            data.update(alias=None)

        if field.is_relation and hasattr(field, "get_attname"):
            data["alias"] = field.get_attname()

        self.__dict__ = data


def convert_django_field_with_choices(
    field: Field,
    *,
    registry: SchemaRegister,
    depth: int = 0,
    skip_registry: bool = False,
) -> Tuple[Type, FieldInfo]:
    converted = convert_django_field(
        field, registry=registry, depth=depth, skip_registry=skip_registry
    )
    return converted


@singledispatch
def convert_django_field(field: Field, **kwargs: Any) -> Tuple[Type, FieldInfo]:
    raise Exception(
        "Don't know how to convert the Django field %s (%s)" % (field, field.__class__)
    )


@no_type_check
def create_m2m_link_type(type_: Type[TModel]) -> Type[TModel]:
    class M2MLink(type_):  # type: ignore
        @classmethod
        def __get_validators__(cls):
            yield cls.validate

        @classmethod
        def validate(cls, v):
            return v.pk

    return M2MLink


@no_type_check
def construct_related_field_schema(
    field: Field, *, registry: SchemaRegister, depth: int, skip_registry=False
) -> Tuple[Type["ModelSchema"], FieldInfo]:
    # create a sample config and return the type
    model = field.related_model
    schema = SchemaFactory.create_schema(
        model, depth=depth - 1, registry=registry, skip_registry=skip_registry
    )
    default = ...
    if not field.concrete and field.auto_created or field.null:
        default = None
    if isinstance(field, models.ManyToManyField):
        schema = List[schema]  # type: ignore

    return (
        schema,
        FieldInfo(
            default=default,
            description=force_str(
                getattr(field, "help_text", field.verbose_name)
            ).strip(),
            title=field.verbose_name.title(),
        ),
    )


@no_type_check
def construct_relational_field_info(
    field: Field,
    *,
    registry: SchemaRegister,
    depth: int = 0,
    __module__: str = __name__,
) -> Tuple[Type, FieldInfo]:
    default: Any = ...
    field_props = FieldConversionProps(field)

    inner_type, field_info = convert_django_field(
        field.related_model._meta.pk, registry=registry, depth=depth
    )

    if not field.concrete and field.auto_created or field.null:
        default = None

    python_type = inner_type
    if field.one_to_many or field.many_to_many:
        m2m_type = create_m2m_link_type(inner_type)
        python_type = List[m2m_type]  # type: ignore

    field_info = FieldInfo(
        default=default,
        alias=field_props.alias,
        default_factory=None,
        title=field_props.title,
        description=field_props.description,
        max_length=None,
    )
    return python_type, field_info


@no_type_check
def construct_field_info(
    python_type: type,
    field: Field,
    depth: int = 0,
    __module__: str = __name__,
    is_custom_type: bool = False,
) -> Tuple[Type, FieldInfo]:
    default = ...
    default_factory = None

    field_props = FieldConversionProps(field)

    if field.choices:
        choices = list(get_choices(field.choices))
        named_choices = [(c[2], c[1]) for c in choices]
        python_type = Enum(  # type: ignore
            f"{field.name.title().replace('_', '')}Enum",
            named_choices,
            module=__module__,
        )
        is_custom_type = True

    if field.has_default():
        if callable(field.default):
            default_factory = field.default
        elif isinstance(field.default, Enum):
            default = field.default.value
        else:
            default = field.default
    elif field_props.blank or field_props.is_null:
        default = None

    if default_factory:
        default = Undefined

    return (
        python_type,
        FieldInfo(
            default=default,
            alias=field_props.alias,
            default_factory=default_factory,
            title=field_props.title,
            description=field_props.description,
            max_length=None if is_custom_type else field_props.max_length,
        ),
    )


@no_type_check
@convert_django_field.register(models.CharField)
@convert_django_field.register(models.TextField)
@convert_django_field.register(models.SlugField)
@convert_django_field.register(models.GenericIPAddressField)
@convert_django_field.register(models.FileField)
@convert_django_field.register(models.FilePathField)
def convert_field_to_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(str, field)


@no_type_check
@convert_django_field.register(models.EmailField)
def convert_field_to_email_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(EmailStr, field, is_custom_type=True)


@no_type_check
@convert_django_field.register(models.URLField)
def convert_field_to_url_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(AnyUrl, field, is_custom_type=True)


@no_type_check
@convert_django_field.register(models.AutoField)
def convert_field_to_id(field: Field, **kwargs: DictStrAny) -> Tuple[Type, FieldInfo]:
    return construct_field_info(int, field)


@no_type_check
@convert_django_field.register(models.UUIDField)
def convert_field_to_uuid(field: Field, **kwargs: DictStrAny) -> Tuple[Type, FieldInfo]:
    return construct_field_info(UUID, field)


@no_type_check
@convert_django_field.register(models.PositiveIntegerField)
@convert_django_field.register(models.PositiveSmallIntegerField)
@convert_django_field.register(models.SmallIntegerField)
@convert_django_field.register(models.BigIntegerField)
@convert_django_field.register(models.IntegerField)
def convert_field_to_int(field: Field, **kwargs: DictStrAny) -> Tuple[Type, FieldInfo]:
    return construct_field_info(int, field)


@no_type_check
@convert_django_field.register(models.BooleanField)
def convert_field_to_boolean(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(bool, field)


@no_type_check
@convert_django_field.register(models.NullBooleanField)
def convert_field_to_null_boolean(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(bool, field)


@no_type_check
@convert_django_field.register(models.DecimalField)
@convert_django_field.register(models.FloatField)
@convert_django_field.register(models.DurationField)
def convert_field_to_float(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(float, field)


@no_type_check
@convert_django_field.register(models.DateTimeField)
def convert_datetime_to_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(datetime.datetime, field)


@no_type_check
@convert_django_field.register(models.DateField)
def convert_date_to_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(datetime.date, field)


@no_type_check
@convert_django_field.register(models.TimeField)
def convert_time_to_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_field_info(datetime.time, field)


@no_type_check
@convert_django_field.register(models.OneToOneRel)
def convert_one_to_one_field_to_django_model(
    field: Field, registry=None, depth=0, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    return construct_relational_field_info(field, registry=registry, depth=depth)


@no_type_check
@convert_django_field.register(models.ManyToManyField)
@convert_django_field.register(models.ManyToManyRel)
@convert_django_field.register(models.ManyToOneRel)
def convert_field_to_list_or_connection(
    field: Field, registry=None, depth=0, skip_registry=False, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    if depth > 0:
        return construct_related_field_schema(
            field, depth=depth, registry=registry, skip_registry=skip_registry
        )
    return construct_relational_field_info(field, registry=registry, depth=depth)


@no_type_check
@convert_django_field.register(models.OneToOneField)
@convert_django_field.register(models.ForeignKey)
def convert_field_to_django_model(
    field: Field,
    registry: Optional[SchemaRegister] = None,
    depth: int = 0,
    skip_registry: bool = False,
    **kwargs: DictStrAny,
) -> Tuple[Type, FieldInfo]:
    if depth > 0:
        return construct_related_field_schema(
            field,
            depth=depth,
            registry=registry or global_registry,
            skip_registry=skip_registry,
        )
    return construct_relational_field_info(field, registry=registry, depth=depth)


@no_type_check
@convert_django_field.register(ArrayField)
def convert_postgres_array_to_list(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    inner_type, field_info = convert_django_field(field.base_field)
    if not isinstance(inner_type, list):
        inner_type = List[inner_type]  # type: ignore
    return inner_type, field_info


@no_type_check
@convert_django_field.register(HStoreField)
@convert_django_field.register(JSONField)
def convert_postgres_field_to_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    python_type = Json
    if field.null:
        python_type = Optional[Json]
    return construct_field_info(python_type, field)


@no_type_check
@convert_django_field.register(RangeField)
def convert_postgres_range_to_string(
    field: Field, **kwargs: DictStrAny
) -> Tuple[Type, FieldInfo]:
    inner_type, field_info = convert_django_field(field.base_field)
    if not isinstance(inner_type, list):
        inner_type = List[inner_type]  # type: ignore
    return inner_type, field_info



import warnings
from typing import TYPE_CHECKING, Any

from pydantic.utils import is_valid_field

from ..errors import ConfigError

if TYPE_CHECKING:
    from pydantic.typing import DictStrAny

__all__ = ["compute_field_annotations"]


def compute_field_annotations(
    namespace: "DictStrAny",
    **field_definitions: Any,
) -> "DictStrAny":

    fields = {}
    annotations = {}

    for f_name, f_def in field_definitions.items():
        if not is_valid_field(f_name):
            warnings.warn(
                f'fields may not start with an underscore, ignoring "{f_name}"',
                RuntimeWarning,
            )
        if isinstance(f_def, tuple):
            try:
                f_annotation, f_value = f_def
            except ValueError as e:
                raise ConfigError(
                    "field definitions should either be a tuple of (<type>, <default>) or just a "
                    "default value, unfortunately this means tuples as "
                    "default values are not allowed"
                ) from e
        else:
            f_annotation, f_value = None, f_def

        if f_annotation:
            annotations[f_name] = f_annotation
        fields[f_name] = f_value

    namespace.update(**{"__annotations__": annotations})
    namespace.update(fields)

    return namespace



import json

import pytest
from django.db import models
from pydantic import ValidationError

from ninja_schema import ModelSchema

SEMESTER_CHOICES = (
    ("1", "One"),
    ("2", "Two"),
    ("3", "Three"),
)


class Student(models.Model):
    semester = models.CharField(max_length=20, choices=SEMESTER_CHOICES, default="1")


class StudentEmail(models.Model):
    email = models.EmailField(null=False, blank=False)


class TestCustomFields:
    def test_enum_field(self):
        class StudentSchema(ModelSchema):
            class Config:
                model = Student
                include = "__all__"

        print(json.dumps(StudentSchema.schema(), sort_keys=False, indent=4))
        assert StudentSchema.schema() == {
            "title": "StudentSchema",
            "type": "object",
            "properties": {
                "id": {"title": "Id", "extra": {}, "type": "integer"},
                "semester": {
                    "title": "Semester",
                    "default": "1",
                    "allOf": [{"$ref": "#/definitions/SemesterEnum"}],
                },
            },
            "definitions": {
                "SemesterEnum": {
                    "title": "SemesterEnum",
                    "description": "An enumeration.",
                    "enum": ["1", "2", "3"],
                }
            },
        }
        schema_instance = StudentSchema(semester="1")
        assert str(schema_instance.json()) == '{"id": null, "semester": "1"}'
        with pytest.raises(ValidationError):
            StudentSchema(semester="something")

    def test_email_field(self):
        class StudentEmailSchema(ModelSchema):
            class Config:
                model = StudentEmail
                include = "__all__"

        print(json.dumps(StudentEmailSchema.schema(), sort_keys=False, indent=4))
        assert StudentEmailSchema.schema() == {
            "title": "StudentEmailSchema",
            "type": "object",
            "properties": {
                "id": {"title": "Id", "extra": {}, "type": "integer"},
                "email": {"title": "Email", "type": "string", "format": "email"},
            },
            "required": ["email"],
        }
        assert (
            str(StudentEmailSchema(email="email@example.com").json())
            == '{"id": null, "email": "email@example.com"}'
        )
        with pytest.raises(ValidationError):
            StudentEmailSchema(email="emailexample.com")



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['ninja_schema',
 'ninja_schema.orm',
 'ninja_schema.orm.utils',
 'ninja_schema.pydanticutils']

package_data = \
{'': ['*']}

install_requires = \
['Django >=3.1', 'pydantic', 'pydantic[email]']

extras_require = \
{'test': ['pytest',
          'pytest-cov',
          'pytest-django',
          'pytest-asyncio',
          'black',
          'isort',
          'flake8',
          'mypy',
          'django-stubs']}

setup(name='ninja-schema',
      version='0.12.2',
      description='Django Schema - Builds Pydantic Schemas from Django Models',
      author='Ezeudoh Tochukwu',
      author_email='tochukwu.ezeudoh@gmail.com',
      url='https://github.com/eadwinCode/ninja-schema',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )



from typing import TYPE_CHECKING, List, Optional, Type, Union, cast

from django.db.models import Model

from ..errors import ConfigError
from ..types import DictStrAny
from .schema_registry import SchemaRegister, registry as schema_registry

if TYPE_CHECKING:
    from .model_schema import ModelSchema
    from .schema import Schema

__all__ = [
    "SchemaFactory",
]


class SchemaFactory:
    @classmethod
    def get_model_config(cls, **kwargs: DictStrAny) -> Type:
        class Config:
            pass

        for key, value in kwargs.items():
            setattr(Config, key, value)
        return Config

    @classmethod
    def create_schema(
        cls,
        model: Type[Model],
        *,
        registry: SchemaRegister = schema_registry,
        name: str = "",
        depth: int = 0,
        fields: Optional[List[str]] = None,
        exclude: Optional[List[str]] = None,
        skip_registry: bool = False
    ) -> Union[Type["ModelSchema"], Type["Schema"], None]:
        from .model_schema import ModelSchema

        name = name or model.__name__

        if fields and exclude:
            raise ConfigError("Only one of 'include' or 'exclude' should be set.")

        schema = registry.get_model_schema(model)
        if schema:
            return schema

        model_config_kwargs = dict(
            model=model,
            include=fields,
            exclude=exclude,
            skip_registry=skip_registry,
            depth=depth,
            registry=registry,
        )
        model_config = cls.get_model_config(**model_config_kwargs)  # type: ignore

        attrs = dict(Config=model_config)

        new_schema = type(name, (ModelSchema,), attrs)
        new_schema = cast(Type[ModelSchema], new_schema)
        if not skip_registry:
            registry.register_model(model, new_schema)
        return new_schema



from django.db import models


class Category(models.Model):
    name = models.CharField(max_length=100)
    start_date = models.DateField()
    end_date = models.DateField()


class Event(models.Model):
    title = models.CharField(max_length=100)
    category = models.OneToOneField(
        Category, null=True, blank=True, on_delete=models.SET_NULL
    )
    start_date = models.DateField(auto_now=True)
    end_date = models.DateField(auto_now_add=True)

    def __str__(self):
        return self.title


class Client(models.Model):
    key = models.CharField(max_length=20, unique=True)
